import { Address, Uint128, ContractInfo, TokenType, Fee, ViewingKey } from './core';
import { SmartContract } from './contract';
import { SigningCosmWasmClient, CosmWasmClient, ExecuteResult } from 'secretjs';
export declare class TokenSaleConfig {
    /**
     * The token that will be used to buy the SNIP20.
     */
    readonly input_token: TokenType;
    /**
     * The total amount that each participant is allowed to buy.
     */
    readonly max_allocation: Uint128;
    /**
     * The minimum amount that each participant is allowed to buy.
     */
    readonly min_allocation: Uint128;
    /**
     * The maximum number of participants allowed.
     */
    readonly max_seats: number;
    /**
     * The price for a single token.
     */
    readonly rate: Uint128;
    readonly sold_token: ContractInfo;
    /**
     * The addresses that are eligible to participate in the sale.
     */
    readonly whitelist: Address[];
    constructor(
    /**
     * The token that will be used to buy the SNIP20.
     */
    input_token: TokenType, 
    /**
     * The total amount that each participant is allowed to buy.
     */
    max_allocation: Uint128, 
    /**
     * The minimum amount that each participant is allowed to buy.
     */
    min_allocation: Uint128, 
    /**
     * The maximum number of participants allowed.
     */
    max_seats: number, 
    /**
     * The price for a single token.
     */
    rate: Uint128, sold_token: ContractInfo, 
    /**
     * The addresses that are eligible to participate in the sale.
     */
    whitelist: Address[]);
}
export interface SaleInfo {
    /**
     * The token that is used to buy the sold SNIP20.
     */
    input_token: TokenType;
    /**
     * The total amount that each participant is allowed to buy.
     */
    max_allocation: Uint128;
    /**
     * The maximum number of participants allowed.
     */
    max_seats: number;
    /**
     * Number of participants currently.
     */
    taken_seats: number;
    /**
     * The minimum amount that each participant is allowed to buy.
     */
    min_allocation: Uint128;
    /**
     * The conversion rate at which the token is sold.
     */
    rate: Uint128;
    /**
     * The token that is being sold.
     */
    sold_token: ContractInfo;
    /**
     * Sale start time.
     */
    start?: number | null;
    /**
     * Sale end time.
     */
    end?: number | null;
}
export interface SaleStatus {
    available_for_sale: Uint128;
    is_active: boolean;
    total_allocation: Uint128;
}
export declare class IdoContract extends SmartContract {
    readonly address: Address;
    readonly signing_client: SigningCosmWasmClient;
    readonly client?: CosmWasmClient | undefined;
    private input_token?;
    constructor(address: Address, signing_client: SigningCosmWasmClient, client?: CosmWasmClient | undefined);
    swap(amount: Uint128, recipient?: Address | null, fee?: Fee): Promise<ExecuteResult>;
    refund(recipient?: Address | null, fee?: Fee): Promise<ExecuteResult>;
    claim(recipient?: Address | null, fee?: Fee): Promise<ExecuteResult>;
    add_addresses(addresses: Address[], fee?: Fee): Promise<ExecuteResult>;
    activate(sale_amount: Uint128, end_time: number, start_time?: number, fee?: Fee): Promise<ExecuteResult>;
    get_balance(key: ViewingKey, address?: Address): Promise<Uint128>;
    get_sale_info(): Promise<SaleInfo>;
    get_sale_status(): Promise<SaleStatus>;
}
